/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.security.impl;

import java.security.Principal;
import java.security.acl.Group;
import java.util.*;

import javax.security.auth.Subject;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.exception.ExoServiceException;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.security.SecurityService;
import org.exoplatform.services.security.SubjectEventListener;
import org.exoplatform.services.security.jaas.JAASGroup;

import edu.yale.its.tp.cas.proxy.ProxyTicketReceptor;

/**
 * Created y the eXo platform team User: Benjamin Mestrallet Date: 28 avr. 2004
 */
public class SecurityServiceImpl implements SecurityService {

  private Map subjects;
  private Log log_;
  private OrganizationService orgService_;
  private String authentication_;
  private String SSOAuthentication_;

  public SecurityServiceImpl(LogService logService,
      OrganizationService organizationService, InitParams params) {
    log_ = logService.getLog("org.exoplatform.services.security");
    orgService_ = organizationService;
    subjects = new HashMap();
    ValueParam param = params.getValueParam("security.authentication");
    authentication_ = null;
    SSOAuthentication_ = null;
    if (param != null)
      authentication_ = param.getValue();
    if (authentication_ == null || authentication_.equals(""))
      authentication_ = SecurityService.STANDALONE_AUTHENTICATION;
    if (SecurityService.SSO_AUTHENTICATION.equals(authentication_)) {
      param = params.getValueParam("security.sso-authentication");
      if (param != null)
        SSOAuthentication_ = param.getValue();
      if (SSOAuthentication_ == null || SSOAuthentication_.equals("")) {
        SSOAuthentication_ = null;
        authentication_ = SecurityService.STANDALONE_AUTHENTICATION;
      }
    }
  }

  public boolean authenticate(String login, String password) throws Exception {   
    if (password == null || "".equals(password)) {
      log_.debug("password must not be null or empty");
      throw new ExoServiceException("password must not be null or empty");
    }
    return orgService_.getUserHandler().authenticate(login, password);
  }

  public void setUpAndCacheSubject(String userName, Subject value) throws ExoServiceException {
    Set principals = value.getPrincipals();
    principals.add(new UserPrincipalImpl(userName));
    Collection groups = null;
    try {
      groups = orgService_.getGroupHandler().findGroupsOfUser(userName);
    } catch (Exception e) {
      throw new ExoServiceException(e);
    }
    Group roleGroup = new JAASGroup(JAASGroup.ROLES);
    for (Iterator iter = groups.iterator(); iter.hasNext();) {
      org.exoplatform.services.organization.Group group = 
        (org.exoplatform.services.organization.Group) iter.next();
      String groupId = group.getId();
      String[] splittedGroupName = StringUtils.split(groupId, "/");
      roleGroup.addMember(new RolePrincipalImpl(splittedGroupName[0]));
    }
    value.getPrincipals().add(roleGroup);
    subjects.put(userName, value);
  }
  
  //Use this for  tomcat 5.5.x
  public void setUpAndCacheSubjectTomcat55(String userName, Subject value) throws ExoServiceException {
    Set principals = value.getPrincipals();
    principals.add(new UserPrincipalImpl(userName));
    Collection groups = null;
    try {
      groups = orgService_.getGroupHandler().findGroupsOfUser(userName);
    } catch (Exception e) {
      throw new ExoServiceException(e);
    }
    for (Iterator iter = groups.iterator(); iter.hasNext();) {
      org.exoplatform.services.organization.Group group = 
        (org.exoplatform.services.organization.Group) iter.next();
      String groupId = group.getId();
      String[] splittedGroupName = StringUtils.split(groupId, "/");
      value.getPrincipals().add(new RolePrincipalImpl(splittedGroupName[0]));
    }
    //value.getPrincipals().add(roleGroup);
    subjects.put(userName, value);
  }

  public boolean isUserInRole(String userName, String role) {
    Subject subject = (Subject) subjects.get(userName);
    if (subject == null) return false;
    Set roleGroups = subject.getPrincipals(Group.class);
    for (Iterator iter = roleGroups.iterator(); iter.hasNext();) {
      Group roleGroup = (Group) iter.next();
      Enumeration e = roleGroup.members();
      while (e.hasMoreElements()) {
        Principal rolePrincipal = (Principal) e.nextElement();
        if (rolePrincipal.getName().equals(role))
          return true;
      }
    }
    return false;
  }

  public boolean hasMembershipInGroup(String userId, String membershipName,
      String groupName) {
    Iterator groups;
    try {
      groups = orgService_.getGroupHandler().findGroupsOfUser(userId)
          .iterator();
    } catch (Exception e) {
      return false;
    }
    if ("*".equals(membershipName)) {
      while (groups.hasNext()) {
        org.exoplatform.services.organization.Group group = (org.exoplatform.services.organization.Group) groups.next();
        if (groupName.equals(group.getId()))
          return true;
      }
    } else {
      while (groups.hasNext()) {
        org.exoplatform.services.organization.Group group = (org.exoplatform.services.organization.Group) groups.next();
        try {
          Iterator memberships = orgService_.getMembershipHandler()
              .findMembershipsByUserAndGroup(userId, group.getId()).iterator();
          while (memberships.hasNext()) {
            Membership membership = (Membership) memberships.next();
            if (membership.getMembershipType().equals(membershipName))
              return true;
          }
        } catch (Exception e) {
          return false;
        }
      }
    }
    return false;
  }

  public boolean hasMembershipInGroup(String user, String roleExpression) {
    if("*".equals(roleExpression))
      return true;
    String membershipName = roleExpression.substring(0, roleExpression
        .indexOf(":"));
    String groupName = roleExpression
        .substring(roleExpression.indexOf(":") + 1);
    return hasMembershipInGroup(user, membershipName, groupName);
  }

  public Subject getSubject(String userName) {
    log_.debug("get subject for user " + userName);
    return (Subject) subjects.get(userName);
  }

  public void removeSubject(String userName) {
    log_.debug("remove subject for user " + userName);
    subjects.remove(userName);
  }

  public void addSubjectEvenetListener(SubjectEventListener subjectEventListener) {
    // To change body of implemented methods use File | Settings | File
    // Templates.
  }

  public String getSSOAuthentication() {
    return SSOAuthentication_;
  }

  public boolean isSSOAuthentication() {
    return SecurityService.SSO_AUTHENTICATION.equals(authentication_);
  }

  public boolean isStandaloneAuthentication() {
    return SecurityService.STANDALONE_AUTHENTICATION.equals(authentication_);
  }

  public String getProxyTicket(String userName, String urlOfTargetService)
      throws Exception {
    if (this.isSSOAuthentication()) {
      if (SecurityService.CAS_AUTHENTICATION.equals(SSOAuthentication_)) {
        Iterator iter = this.getSubject(userName).getPrivateCredentials()
            .iterator();
        if (!iter.hasNext())
          return null;
        // TODO : test CAS proxy failure for invalid or expired PGTs
        return ProxyTicketReceptor.getProxyTicket((String) iter.next(),
            urlOfTargetService);
      } else
        throw new ExoServiceException(
            "No proxy authentication feature available for "
                + SSOAuthentication_ + " SSO authentication !");

    } else
      throw new ExoServiceException(
          "Portal is configured for standalone authentication. "
              + "No proxy authentication feature available !");
  }

  public Log getLog() {
    return log_;
  }
}
